#!/bin/sh

. /images/install.conf
. $PROGRESS

PATH=/bin:/sbin:/usr/bin:/usr/sbin

#
# internal parameters
#
DEV_QSPI=/dev/mtdblock0
DEV_EMMC=/dev/mmcblk2
DEV_EMMC_BOOT=${DEV_EMMC}boot0
EMMC_BOOT_RO=/sys/block/mmcblk2boot0/force_ro
KERNEL_EMMC_PATH=${DEV_EMMC}p1
USERLAND_EMMC_PATH=${DEV_EMMC}p2
RECOVERY_EMMC_PATH=${DEV_EMMC}p3

STEP=0

print_serial() {
	echo $*
}

print_step() {
	STEP=$(($STEP+1))
	print_serial "=== Step.${STEP} $* ==="
}

error_exit() {
	print_serial "##### $*"
	progress_ng
	exit 2
}

emmc_format() {
# fdisk -l /dev/mmcblk2
#
# Disk /dev/mmcblk2: 3.6 GiB, 3825205248 bytes, 7471104 sectors
# Units: sectors of 1 * 512 = 512 bytes
# Sector size (logical/physical): 512 bytes / 512 bytes
# I/O size (minimum/optimal): 512 bytes / 512 bytes
# Disklabel type: dos
# Disk identifier: 0x37b82311
#
# Device         Boot  Start     End Sectors  Size Id Type
# /dev/mmcblk2p1        2048   67583   65536   32M  b W95 FAT32
# /dev/mmcblk2p2      329728 7471103 7141376  3.4G 83 Linux
# /dev/mmcblk2p3       67584  329727  262144  128M  b W95 FAT32
	fdisk ${DEV_EMMC} <<EOF
o
w
EOF

	loop=100 #10second
	while true
	do
	    ls ${DEV_EMMC}p* > /dev/null 2>&1
	    if [ $? -ne 0 ]; then
		break
	    fi

	    sleep 0.1
	    loop=$(expr $loop - 1)
	    if [ $loop -eq 0 ]; then
		echo $(ls ${DEV_EMMC}p*)" is busy"
		exit 1
	    fi
	done

	fdisk ${DEV_EMMC} <<EOF
n
p
1

+32M
n
p
3

+128M
n
p
2


t
1
b
t
2
83
t
3
b
w
EOF

	udevtrigger
	udevsettle

	mkfs.vfat ${KERNEL_EMMC_PATH}
	mke2fs -t ext4 ${USERLAND_EMMC_PATH}
	mkfs.vfat ${RECOVERY_EMMC_PATH}

}

clear_bootloader() {
	dd if=/dev/zero of=$1
	sync
}

setup_bootloader() {
	if [ ! -z "${BOOTLOADER_IMAGE_MD5}" ]; then
		echo "${BOOTLOADER_IMAGE_MD5}  ${BOOTLOADER_IMAGE}" | md5sum -c
		[ $? -eq 0 ] || return 1
	fi

	if [ ! -f "${BOOTLOADER_IMAGE}" ]; then
		echo "file not found ${BOOTLOADER_IMAGE}"
		return 1
	fi

	if [ -e ${DEV_QSPI} ]; then
		clear_bootloader ${DEV_QSPI}
		dd if=${BOOTLOADER_IMAGE} of=${DEV_QSPI}
	fi

	if [ -e ${DEV_EMMC_BOOT} ]; then
		echo 0 > ${EMMC_BOOT_RO}
		clear_bootloader ${DEV_EMMC_BOOT}
		dd if=${BOOTLOADER_IMAGE} of=${DEV_EMMC_BOOT} bs=1k skip=1 seek=1
		sync
		echo 1 > ${EMMC_BOOT_RO}
	fi
}

extract_file() {
	local DEVICE=$1
	local FS=$2
	local FILE=$3
	local MD5=$4

	if [ ! -z "${FILE}" ]; then
		if [ ! -z "${MD5}" ]; then
			echo "${MD5}  ${FILE}" | md5sum -c
			[ $? -eq 0 ] || return 1
		fi

		mkdir -p /tmp/${DEVICE}
		mount -t ${FS} /dev/${DEVICE} /tmp/${DEVICE}
		tar xf ${FILE} -C /tmp/${DEVICE}
		sync
		umount /tmp/${DEVICE}
	else
		echo "file not found ${FILE}"
		return 1
	fi
}

copy_file() {
	local DEVICE=$1
	local FS=$2
	local FILE=$3
	local MD5=$4
	local TO=$5

	if [ ! -z "${FILE}" ]; then
		if [ ! -z "${MD5}" ]; then
			echo "${MD5}  ${FILE}" | md5sum -c
			[ $? -eq 0 ] || return 1
		fi

		mkdir -p /tmp/${DEVICE}
		mount -t ${FS} /dev/${DEVICE} /tmp/${DEVICE}
		mkdir -p /tmp/${DEVICE}/${TO%/*}
		cp ${FILE} /tmp/${DEVICE}/${TO}
		sync
		umount /tmp/${DEVICE}
	else
		echo "file not found ${FILE}"
		return 1
	fi
}

loop() {
	while :
	do
		:
	done
}

#
# main
#
print_serial "*+*+* Install Start!! *+*+*"
progress_running

print_step "Board Update"
	${BOARD_UPDATE_TOOL} 2>&1
	[ $? -ne 0 ] && error_exit "Board Update Error"

print_step "Setup Bootloader"
	setup_bootloader
sleep 5

print_step "Formatting eMMC"
		emmc_format
		fdisk -l /dev/mmcblk2

print_step "Setup Partition"
	extract_file mmcblk2p2 ext4 \
		"${ROOTFS_ARCHIVE}" "${ROOTFS_ARCHIVE_MD5}"
	[ $? -ne 0 ] && error_exit "MD5 error: ${ROOTFS_ARCHIVE}"
	copy_file mmcblk2p1 vfat \
		"${KERNEL_IMAGE}" "${KERNEL_IMAGE_MD5}" .
	[ $? -ne 0 ] && error_exit "MD5 error: ${KERNEL_IMAGE}"
	copy_file mmcblk2p1 vfat \
		"${DEVICETREE_IMAGE}" "${DEVICETREE_IMAGE_MD5}" .
	[ $? -ne 0 ] && error_exit "MD5 error: ${DEVICETREE_IMAGE}"

print_step "sync"
	sync

print_serial "*+*+* Install Completed!! *+*+*"
progress_ok

# loop to avoid auto reboot
loop
